@@ -27,6 +27,50 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: thread.h,v $
+ * Revision 1.38.2.1  2006/01/29 23:34:53  csoutheren
+ * Backported thread destruction patch from Derek Smithies
+ *
+ * Revision 1.39  2006/01/29 22:35:46  csoutheren
+ * Added fix for thread termination problems on SMP machines
+ * Thanks to Derek Smithies
+ *
+ * Revision 1.38  2006/01/11 22:27:44  dereksmithies
+ * Add extra comments describing the usage of Resume() in the constructor of a
+ * class descended of PThread
+ *
+ * Revision 1.37  2005/11/30 12:47:38  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.36  2005/11/25 03:43:47  csoutheren
+ * Fixed function argument comments to be compatible with Doxygen
+ *
+ * Revision 1.35  2003/10/08 21:39:34  dereksmithies
+ * Add a #define to cope with backward compatability issues for PThreadIdentifier
+ * Thanks to Andrey S Pankov and Craig Southeren for their input.
+ *
+ * Revision 1.34  2003/09/17 05:41:59  csoutheren
+ * Removed recursive includes
+ *
+ * Revision 1.33  2003/09/17 01:18:02  csoutheren
+ * Removed recursive include file system and removed all references
+ * to deprecated coooperative threading support
+ *
+ * Revision 1.32  2002/10/04 04:33:27  robertj
+ * Added functions for getting operating system thread identifier values.
+ *
+ * Revision 1.31  2002/09/16 01:08:59  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.30  2002/06/27 06:44:28  robertj
+ * Changed "" to PString::Empty() where assigning to PString.
+ *
+ * Revision 1.29  2002/04/24 01:49:22  robertj
+ * Fixed error in PTRACE_BLOCk nesting level to now work when no tracing enabled.
+ *
+ * Revision 1.28  2002/04/24 01:09:56  robertj
+ * Fixed problem with PTRACE_BLOCK indent level being correct across threads.
+ *
  * Revision 1.27  2001/09/10 02:51:22  robertj
  * Major change to fix problem with error codes being corrupted in a
  *   PChannel when have simultaneous reads and writes in threads.
@@ -110,8 +154,10 @@
  *
  */
 
+#ifndef _PTHREAD
+#define _PTHREAD
 
-#ifdef __GNUC__
+#ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
@@ -121,6 +167,9 @@
 
 class PSemaphore;
 
+#define PThreadIdentifer PThreadIdentifier
+
+typedef P_THREADIDENTIFIER PThreadIdentifier;
 
 ///////////////////////////////////////////////////////////////////////////////
 // PThread
@@ -133,26 +182,10 @@ class PSemaphore;
    This is reflected in this library by the #PProcess# class being
    descended from the PThread class.
 
-   The implementation of a thread is platform dependent. Not all platforms
-   support concurrent threads within a process or even concurrent processes!
-   For example, MS-DOS has no form of multi-threading or multi-processing,
-   Microsoft Windows has a cooperative multi-processing but no multi-threading.
-   Unix has full pre-emptive multi-processing but most cannot do multiple
-   threads within that process while some Unix systems and Windows NT have
-   full preemptive proceses and threads.
-
-   If a platform does not directly support multiple threads, the library will
-   them using a cooperative co-routine technique. This requires that each
-   thread of execution within a process, voluntarily yields control to other
-   threads. This will occur if the thread is blocked inside an I/O function
-   on a #PChannel# or when the #PThread::Yield()# function is
-   explicitly called.
-   
-   Note that this is {\bf cooperative}. An endless loop will stop all
-   threads in a process, possibly all processes on some platforms. If a
-   lengthy operation is to take place that does not involve blocking I/O,
-   eg pure computation or disk file I/O, then it is the responsiblity of the
-   programmer to assure enough yielding for background threads to execute.
+   The implementation of a thread is platform dependent, but it is
+   assumed that the platform has some support for native threads.
+   Previous versions of PWLib has some support for co-operative
+   threads, but this has been removed
  */
 class PThread : public PObject
 {
@@ -208,20 +241,16 @@ class PThread : public PObject
        may be freed using the delete operator as soon as the thread is
        terminated or executes to completion (usually the latter).
 
-       The stack size specified is {\bf not} simply in bytes. It is a value
-       that is multiplied by a factor into bytes depending on the target
-       platform. For example a Unix system with a RISC processor may use
-       significantly more stack than an MS-DOS platform. These sizes are
-       normalised to the "stack factor" provided here. For some platforms, eg
-       Windows NT, the stack size is only an initial size and the stack will
-       automatically be increased as required.
+       The stack size argument retained only for source code compatibility for
+       previous implementations. It is not used in the current code and
+       may be removed in subsequent versions.
      */
     PThread(
-      PINDEX stackSize,                 /// Size of stack to use for thread.
+      PINDEX ,                 ///< Not used - previously stack size
       AutoDeleteFlag deletion = AutoDeleteThread,
-        /// Automatically delete PThread instance on termination of thread.
-      Priority priorityLevel = NormalPriority,  /// Initial priority of thread.
-      const PString & threadName = "" /// The name of the thread (for Debug/Trace)
+        ///< Automatically delete PThread instance on termination of thread.
+      Priority priorityLevel = NormalPriority,  ///< Initial priority of thread.
+      const PString & threadName = PString::Empty() ///< The name of the thread (for Debug/Trace)
     );
 
     /** Destroy the thread, this simply calls the #Terminate()# function
@@ -241,7 +270,7 @@ class PThread : public PObject
        polymorphically.
       */
     void PrintOn(
-      ostream & strm    /// Stream to output text representation
+      ostream & strm    ///< Stream to output text representation
     ) const;
   //@}
 
@@ -281,7 +310,7 @@ class PThread : public PObject
      */
     void WaitForTermination() const;
     BOOL WaitForTermination(
-      const PTimeInterval & maxWait  /// Maximum time to wait for termination.
+      const PTimeInterval & maxWait  ///< Maximum time to wait for termination.
     ) const;
 
     /** Suspend or resume the thread.
@@ -297,11 +326,27 @@ class PThread : public PObject
        #Suspend(TRUE)# calls are made.
      */
     virtual void Suspend(
-      BOOL susp = TRUE    /// Flag to suspend or resume a thread.
+      BOOL susp = TRUE    ///< Flag to suspend or resume a thread.
     );
 
     /** Resume thread execution, this is identical to
        #Suspend(FALSE)#.
+
+      The Resume() method may be called from within the constructor of a
+      PThread descendant.  However, the Resume() should be in the
+      constructor of the most descendant class. So, if you have a
+      class B (which is descended of PThread), and a class C (which is
+      descended of B), placing the call to Resume in the constructor of B is
+      unwise.
+
+      If you do place a call to Resume in the constructor, it
+      should be at the end of the constructor, after all the other
+      initialisation in the constructor.
+
+      The reason the call to Resume() should be at the end of the
+      construction process is simple - you want the thread to start
+      when all the variables in the class have been correctly
+      initialised.
      */
     virtual void Resume();
 
@@ -316,14 +361,14 @@ class PThread : public PObject
 
     /// Suspend the current thread for the specified amount of time.
     static void Sleep(
-      const PTimeInterval & delay   /// Time interval to sleep for.
+      const PTimeInterval & delay   ///< Time interval to sleep for.
     );
 
     /** Set the priority of the thread relative to other threads in the current
        process.
      */
     virtual void SetPriority(
-      Priority priorityLevel    /// New priority for thread.
+      Priority priorityLevel    ///< New priority for thread.
     );
 
     /** Get the current priority of the thread in the current process.
@@ -337,7 +382,7 @@ class PThread : public PObject
         when the thread ends.
      */
     virtual void SetAutoDelete(
-      AutoDeleteFlag deletion = AutoDeleteThread  /// New auto delete setting.
+      AutoDeleteFlag deletion = AutoDeleteThread  ///< New auto delete setting.
     );
 
     /** Reet the flag indicating thread object is to be automatically deleted
@@ -358,12 +403,20 @@ class PThread : public PObject
        current thread name.
      */
     virtual void SetThreadName(
-      const PString & name        /// New name for the thread.
+      const PString & name        ///< New name for the thread.
     );
   //@}
 
   /**@name Miscellaneous */
   //@{
+    /** Get operating system specific thread identifier for this thread.
+      * Note that the return value from these functions is only valid
+      * if called by the owning thread. Calling this function for another
+      * thread that may be terminating is a very bad idea.
+      */
+    virtual PThreadIdentifier GetThreadId() const;
+    static PThreadIdentifier GetCurrentThreadId();
+
     /** User override function for the main execution routine of the thread. A
        descendent class must provide the code that will be executed in the
        thread within this function.
@@ -384,15 +437,11 @@ class PThread : public PObject
      */
     static PThread * Current();
 
-    /** Yield to another thread. If there are no other threads then this
-       function does nothing. Note that on most platforms the threading is
-       cooperative and this function must be called for other threads to run
-       at all. There may be an implicit call to Yield within the I/O functions
-       of #PChannel# classes. This is so when a thread is I/O blocked then
-       other threads can, as far as possible, continue to run.
-       
-       If the platform directly supports multiple threads then this function
-       will do nothing.
+    /** Yield to another thread without blocking.
+        This duplicates the implicit thread yield that may occur on some
+        I/O operations or system calls.
+
+        This may not be implemented on all platforms.
      */
     static void Yield();
 
@@ -401,13 +450,13 @@ class PThread : public PObject
        function defined by the PNotifier in the context of a new thread.
       */
     static PThread * Create(
-      const PNotifier & notifier,     /// Function to execute in thread.
-      INT parameter = 0,              /// Parameter value to pass to notifier.
+      const PNotifier & notifier,     ///< Function to execute in thread.
+      INT parameter = 0,              ///< Parameter value to pass to notifier.
       AutoDeleteFlag deletion = AutoDeleteThread,
-        /// Automatically delete PThread instance on termination of thread.
-      Priority priorityLevel = NormalPriority,  /// Initial priority of thread.
-      const PString & threadName = "", /// The name of the thread (for Debug/Trace)
-      PINDEX stackSize = 10000         /// Stack size on some platforms
+        ///< Automatically delete PThread instance on termination of thread.
+      Priority priorityLevel = NormalPriority,  ///< Initial priority of thread.
+      const PString & threadName = PString::Empty(), ///< The name of the thread (for Debug/Trace)
+      PINDEX stackSize = 10000         ///< Stack size on some platforms
     );
   //@}
 
@@ -417,19 +466,6 @@ class PThread : public PObject
        required due to the bootstrap logic of processes and threads.
      */
 
-#ifndef P_PLATFORM_HAS_THREADS
-    virtual BOOL IsNoLongerBlocked();
-    /* Check if the condition that has blocked a thread in an I/O function,
-       for example, has ceased. This is required by the internal cooperative
-       thread scheduler.
-
-       This function is not present for platforms that support threads.
-       
-       <H2>Returns:</H2>
-       TRUE if the thread is no longer blocked.
-     */
-#endif
-
   private:
     PThread();
     // Create a new thread instance as part of a PProcess class.
@@ -449,141 +485,19 @@ class PThread : public PObject
     // Give the thread a name for debugging purposes.
     PString threadName;
 
-#ifndef P_PLATFORM_HAS_THREADS
-    void AllocateStack(
-      PINDEX stackSize  // Size of the stack to allocate.
-    );
-    /* Allocate the stack for the thread.
-
-       The stack size specified is {\bf not} simply in bytes. It is a value
-       that is multiplied by a factor into bytes depending on the target
-       platform. For example a Unix system with a RISC processor may use
-       significantly more stack than an MS-DOS platform. These sizes are
-       normalised to the "stack factor" provided here. For some platforms, eg
-       Windows NT, the stack size is only an initial size and the stack will
-       automatically be increased as required.
-
-       This function is not present for platforms that support threads.
-     */
-
-    void ClearBlock();
-    /* Clear the blocked thread. This is used by platform dependent code to
-       signal to the common code scheduler that the thread is no longer
-       blocked.
-
-       This function is not present for platforms that support threads.
-     */
-
-    void BeginThread();
-    /* Function to start #Main()# and exit when completed.
-
-       This function is not present for platforms that support threads.
-     */
-
-    virtual void SwitchContext(
-      PThread * from    // Thread being switched from.
-    );
-    /* Do the machinations needed to jump to the current thread. This is a
-       platform dependent function that utilises the standard C
-       #setjmp()# and #longjmp()# functions to implement
-       the co-routines.
-    
-       This function is not present for platforms that support threads.
-     */
-
-
-    // Member fields
-    Priority basePriority;
-    /* The threads priority level, relative to other threads.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    int dynamicPriority;
-    /* The threads priority during this scheduled slice. A thread that has not
-       been scheduled has its dynamic priority increased so that next time
-       the scheduler is looking for a thread to run it has a better chance of
-       executing. Once a thread is executed the dynamic priority is set back
-       to the base priority as set by #SetPriority()#.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    int suspendCount;
-    /* The threads count of calls to #Suspend()# or #Resume()#.
-       If <=0 then can run, if >0 means suspended and is not to be scheduled.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    PTimer sleepTimer;
-    /* Time for thread to remain asleep. Thread is not scheduled while this
-       is running after a #Sleep()# call.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    PSemaphore * blockingSemaphore;
-    /* Semaphore that is blocking this thread.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    PThread * link;
-    /* Link to next thread in circular list. The use of a list rather than
-       priority queues or other sophisticated task queuing technique is to
-       simplify the scheduler. It is expected that any given process will not
-       have a large number of threads, ie less than approximately ten, so the
-       overhead of the list search is small in comparison to the overhead in
-       complex data structures.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    enum {
-      Starting,         // Thread is starting up.
-      Running,          // Thread is the currently executing context.
-      Waiting,          // Thread is waiting to be scheduled.
-      Sleeping,         // Thread is sleeping until sleepTimer is up.
-      Suspended,        // Thread is currently suspended.
-      BlockedIO,        // Thread is currently blocked in I/O.
-      SuspendedBlockIO, // Thread is blocked {\bf and} suspended.
-      BlockedSem,       // Thread is currently blocked by a semaphore.
-      SuspendedBlockSem,// Thread is blocked {\bf and} suspended.
-      Terminating,      // Thread is terminating but has not died yet.
-      Terminated        // Thread has terminated.
-    } status;
-    /* Thread status for scheduler handling.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    jmp_buf context;
-    /* Buffer for context switching.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    char PSTATIC * stackBase;
-    /* Base of stack allocated for the thread. The PSTATIC is for DOS-Windows.
-
-       This variable is not present for platforms that support threads.
-     */
-
-    char PSTATIC * stackTop;
-    /* Top of stack allocated for the thread.
-
-       This variable is not present for platforms that support threads.
-     */
-
-
-    friend class PSemaphore;
-#endif
+  private:
+    unsigned traceBlockIndentLevel;
+    friend class PTrace::Block;
 
 
 // Include platform dependent part of class
-#include <ptlib/thread.h>
+#ifdef _WIN32
+#include "msos/ptlib/thread.h"
+#else
+#include "unix/ptlib/thread.h"
+#endif
 };
 
+#endif // _PTHREAD
 
 // End Of File ///////////////////////////////////////////////////////////////
