@@ -28,6 +28,15 @@
  * Contributor(s): /
  *
  * $Log: sound_alsa.cxx,v $
+ * Revision 1.27.2.5  2006/12/30 22:22:30  dsandras
+ * Fixed possible ALSA crash because of the use of a NULL pointer if
+ * a detected device can not be opened for some reason. (Ekiga report #328753).
+ * (Backport from HEAD).
+ *
+ * Revision 1.27.2.4  2006/10/06 11:57:46  dsandras
+ * Increases storedPeriods to 3 as suggested by Stelian Pop (Ekiga bug #358338)
+ * to fix bad output sound on some soundcards.
+ *
  * Revision 1.27.2.3  2006/05/17 18:41:36  dsandras
  * Backported fix from HEAD.
  *
@@ -135,6 +144,7 @@ PCREATE_SOUND_PLUGIN(ALSA, PSoundChannelALSA)
 
 static PStringToOrdinal playback_devices;
 static PStringToOrdinal capture_devices;
+PMutex dictionaryMutex;
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -158,7 +168,7 @@ PSoundChannelALSA::PSoundChannelALSA (const PString &device,
 void PSoundChannelALSA::Construct()
 {
   frameBytes = 480;
-  storedPeriods = 2;
+  storedPeriods = 3;
   storedSize = storedPeriods * frameBytes;
 
   card_nr = 0;
@@ -184,6 +194,8 @@ void PSoundChannelALSA::UpdateDictionary (Directions dir)
   char *name = NULL;
   char card_id [32];
 
+  PWaitAndSignal m(dictionaryMutex);
+
   if (dir == Recorder) {
 
     stream = SND_PCM_STREAM_CAPTURE;
@@ -233,9 +245,9 @@ void PSoundChannelALSA::UpdateDictionary (Directions dir)
           free (name);
         }
       }
+      snd_ctl_close(handle);
     }
 
-    snd_ctl_close(handle);
     snd_card_next (&card);
   }
 }
@@ -276,10 +288,10 @@ PString PSoundChannelALSA::GetDefaultDevice(Directions dir)
 
 
 BOOL PSoundChannelALSA::Open (const PString & _device,
-                                   Directions _dir,
-                                     unsigned _numChannels,
-                                     unsigned _sampleRate,
-                                     unsigned _bitsPerSample)
+                              Directions _dir,
+                              unsigned _numChannels,
+                              unsigned _sampleRate,
+                              unsigned _bitsPerSample)
 {
   PString real_device_name;
   POrdinalKey *i = NULL;
@@ -295,6 +307,8 @@ BOOL PSoundChannelALSA::Open (const PString & _device,
 
   os_handle = NULL;
 
+  PWaitAndSignal m(device_mutex);
+
   if (_dir == Recorder)
     stream = SND_PCM_STREAM_CAPTURE;
   else
