@@ -25,6 +25,14 @@
  *                 Mark Cooke (mpc@star.sr.bham.ac.uk)
  *
  * $Log: vidinput_v4l.cxx,v $
+ * Revision 1.15.2.5  2007/01/03 22:37:37  dsandras
+ * Backports from HEAD.
+ *
+ * Revision 1.15.2.4  2006/11/28 21:07:03  dsandras
+ * Added a few missing mutexes in order to prevent collection
+ * corruption when the update is called from different threads.
+ * Hopefully fixes Ekiga report #376078.
+ *
  * Revision 1.15.2.3  2006/06/10 15:57:46  dsandras
  * Added HINTS for Logitech Quickcam Communicate thanks to Michael Riepe
  * <michael mr511 de>.
@@ -418,6 +426,7 @@ V4LNames::Update()
   PString      entry;
   PStringList  devlist;
   
+  PWaitAndSignal m(mutex);
   inputDeviceNames.RemoveAll (); // flush the previous run
   if (procvideo.Exists()) {
     if (procvideo.Open(PFileInfo::RegularFile)) {
@@ -526,7 +535,6 @@ PString V4LNames::GetUserFriendly(PString devName)
 {
   PWaitAndSignal m(mutex);
 
-  
   PString result= deviceKey(devName);
   if (result.IsEmpty())
     return devName;
@@ -547,6 +555,8 @@ PString V4LNames::GetDeviceName(PString userName)
 
 void V4LNames::AddUserDeviceName(PString userName, PString devName)
 {
+  PWaitAndSignal m(mutex);
+
   if (userName != devName) { // must be a real userName!
     userKey.SetAt(userName, devName);
     deviceKey.SetAt(devName, userName);
@@ -589,15 +599,16 @@ PStringList V4LNames::GetInputDeviceNames()
   PStringList result;
   for (PINDEX i = 0; i < inputDeviceNames.GetSize(); i++) {
     result += GetUserFriendly (inputDeviceNames[i]);
-    //result += inputDeviceNames[i];
   }
 
   return result;
 }
 
+PMutex creationMutex;
 static 
 V4LNames & GetNames()
 {
+  PWaitAndSignal m(creationMutex);
   static V4LNames names;
   names.Update();
   return names;
