@@ -27,6 +27,26 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: tlib.cxx,v $
+ * Revision 1.76  2005/11/30 12:47:42  csoutheren
+ * Removed tabs, reformatted some code, and changed tags for Doxygen
+ *
+ * Revision 1.75  2005/11/08 10:35:53  csoutheren
+ * Added missing pragma
+ *
+ * Revision 1.74  2005/08/04 18:54:10  csoutheren
+ * Applied fix for bug Bugs item #1244310
+ * Fix problem wioth --disable-openh323 option
+ * Thanks to Michael Manousos
+ *
+ * Revision 1.73  2005/02/22 03:25:43  dereksmithies
+ * Add fix from huangzb@a-star.com.cn to correct a typo. Many thanks.
+ *
+ * Revision 1.72  2005/01/26 05:38:05  csoutheren
+ * Added ability to remove config file support
+ *
+ * Revision 1.71  2005/01/04 08:09:43  csoutheren
+ * Fixed Linux configure problems
+ *
  * Revision 1.70  2004/07/11 07:56:36  csoutheren
  * Applied jumbo VxWorks patch, thanks to Eize Slange
  *
@@ -238,6 +258,7 @@
 #pragma implementation "thread.h"
 #pragma implementation "semaphor.h"
 #pragma implementation "mutex.h"
+#pragma implementation "psync.h"
 #pragma implementation "syncpoint.h"
 #pragma implementation "syncthrd.h"
 
@@ -278,24 +299,26 @@ extern "C" {
 #include "uerror.h"
 
 #if defined(P_HPUX9)
-#define	SELECT(p1,p2,p3,p4,p5)		select(p1,(int *)(p2),(int *)(p3),(int *)(p4),p5)
+#define  SELECT(p1,p2,p3,p4,p5)    select(p1,(int *)(p2),(int *)(p3),(int *)(p4),p5)
 #else
-#define	SELECT(p1,p2,p3,p4,p5)		select(p1,p2,p3,p4,p5)
+#define  SELECT(p1,p2,p3,p4,p5)    select(p1,p2,p3,p4,p5)
 #endif
 
 #if defined(P_SUN4)
 extern "C" void bzero(void *, int);
 extern "C" int select(int width,
-			fd_set *readfds,
-			fd_set *writefds,
-			fd_set *exceptfds,
-			struct timeval *timeout);
+      fd_set *readfds,
+      fd_set *writefds,
+      fd_set *exceptfds,
+      struct timeval *timeout);
 #endif
 
 #ifdef __BEOS__
 #include "OS.h"
 #endif
 
+#include "../common/pglobalstatic.cxx"
+
 
 PString PProcess::GetOSClass()
 {
@@ -419,7 +442,7 @@ PString PProcess::GetUserName() const
   char pnamebuf[1024];
   int len = 1024;
   STATUS gethostresult;
-  gethostresult =::gethostname(pnamebuf,len);	
+  gethostresult =::gethostname(pnamebuf,len);  
   if (gethostresult == OK)
     return PString(pnamebuf,len);
   else
@@ -713,12 +736,16 @@ void PProcess::CommonConstruct()
   tzset();
 #endif
 
+#ifdef P_CONFIG_FILE
   CreateConfigFilesDictionary();
+#endif
 }
 
 void PProcess::CommonDestruct()
 {
+#ifdef P_CONFIG_FILE
   delete configFiles;
+#endif
   configFiles = NULL;
   SetSignals(NULL);
 }
@@ -729,7 +756,7 @@ void PProcess::CommonDestruct()
 extern "C" {
 #include <netinet/in.h>
 #include <rtems/rtems_bsdnet.h>
-	
+  
 
 int socketpair(int d, int type, int protocol, int sv[2])
 {
@@ -744,13 +771,13 @@ int socketpair(int d, int type, int protocol, int sv[2])
     {
         printf("\"Network\" initializing!\n");
         network_status = rtems_bsdnet_initialize_network();
-	if (network_status == 0)
-	    printf("\"Network\" initialized!\n");
-	else
-	{
-	    printf("Error: %s\n", strerror(errno));
-	    return -1;
-	}
+  if (network_status == 0)
+      printf("\"Network\" initialized!\n");
+  else
+  {
+      printf("Error: %s\n", strerror(errno));
+      return -1;
+  }
     }
 
     /* prepare sv */
@@ -768,19 +795,19 @@ int socketpair(int d, int type, int protocol, int sv[2])
 
     if (bind(s, (struct sockaddr *)&addr1, sizeof addr1) < 0) 
     {
-	close(s);
+  close(s);
         return -1;
     }
     if (listen(s, 2) < 0 ) 
     {
-	close(s);
+  close(s);
         return -1;
     }
     
     sv[0] = socket(d, type, protocol);
     if (sv[0] < 0) 
     {
-	close(s);
+  close(s);
         return -1;
     }
     
@@ -791,25 +818,25 @@ int socketpair(int d, int type, int protocol, int sv[2])
 
     if (bind(sv[0], (struct sockaddr *)&addr2, sizeof addr2) < 0)
     {
-	close(s);
-	close(sv[0]);
-	sv[0]=-1;
+  close(s);
+  close(sv[0]);
+  sv[0]=-1;
         return -1;
     }
     if (connect(sv[0], (struct sockaddr *)&addr1, sizeof addr1) < 0)
     {
-	close(s);
-	close(sv[0]);
-	sv[0]=-1;
+  close(s);
+  close(sv[0]);
+  sv[0]=-1;
         return -1;
     }
     
     sv[1] = accept(s, (struct sockaddr *)&addr2, &addrlen);
     if (sv[1] < 0)
     {
-	close(s);
-	close(sv[0]);
-	sv[0]=-1;
+  close(s);
+  close(sv[0]);
+  sv[0]=-1;
         return -1;
     }
 
